#
# Convert the PAMF state table into an SDB rule
#
# Tom Moore, October 26, 1995
#

proc parse {words where} {
    #
    # Recursive routine to figure out what to do with the conditions
    #

    global Var 
    global CurrentElement
    global ElementCount
    global Rule
    global Type

    if {$where == $ElementCount} {
	AppendTerminal
	return
    }

    set first [lindex $words 0]

    #
    # This conversion utility has not been debugged for
    # wildcards yet.  They are position sensitive and may
    # require that the records be sorted into sensible order
    # first.  For now, refuse to tackle them.
    #
    if {$first == "*"} {
	puts "Ooops!  Got a wild card"
	return
    } 

    #
    # Parse the operator and value out of the element
    #
    regexp {([<>=]*)(.*)} $first all op value
    if {$op == {}} {
	set op "=="
    }
    if {[set Type([set where])] == "CODE"} {
	set value "\"$value\""
    }

    #
    # If this is the first time that this column has been visited
    # then output the beginning of the conditional
    #
    if {[string compare [set CurrentElement([set where])] "zzzzzzz"] == 0} then {
	append Rule "[set Var([set where])]?($op,$value,"
	set CurrentElement([set where]) $first

    #
    # Else, if the decision variable value has changed, terminate
    # the previous clause and start another
    #
    } elseif {[string compare [set CurrentElement([set where])] $first] != 0} {
	for {set i [expr $where + 1]} {$i < $ElementCount} {incr i} {
	    append Rule "):0"
	    set CurrentElement($i) "zzzzzzz"
	}
	append Rule ")
"
        for {set i 0} {$i <= $where} {incr i} {append Rule "    "}
        append Rule ":($op,$value,"
	set CurrentElement([set where]) $first
    }

    #
    # Now recusively attack the rest of the list
    #
    parse [lrange $words 1 end] [expr $where +1]
}

proc AppendTerminal {} {
    #
    # Format the assignment portion of the rule
    #

    global Terminal
    global Rule
    global ElementCount

    set space {}
    for {set i 0} {$i < [expr $ElementCount + 1]} {incr i} {
	append space "    "
    }
    append Rule "
[set space]("
    set length [llength $Terminal]
    for {set j 1;set i 0} {$i < $length} {incr j;incr i 4} {
	if {$j > 1} {
	    append Rule "
[set space] "
	}
	append Rule "CODE_[set j]=\"[lindex $Terminal $i]\";"
	append Rule "SITE_[set j]=\"[lindex $Terminal [expr $i + 1]]\";"
	append Rule "Origin_[set j]=date-[lindex $Terminal [expr $i + 2]];"
	append Rule "STKG_[set j]=[lindex $Terminal [expr $i + 3]];"
    }
    append Rule "
[set space] CODE=CODE_1;SITE=SITE_1;origin=Origin_1;1)"
}

proc extractRule {file} {
    #
    # Given a file name, this routine opens the file,
    # scans the column headings, and calls the recursive
    # routine to parse the records.  Returns the entire
    # rule when finished.
    #
    global Var 
    global CurrentElement
    global ElementCount
    global Type
    global Terminal
    global Rule
	
    set Rule {}

    #
    # Open the succession rule file
    #
    set in [open $file "r"]

    #
    # Parse the first record, which contains the column heading
    #
    gets $in record
    regsub -all {[ ]+} $record " " newrecord
    
    set ElementCount 0
    foreach element $newrecord {
	if {$element == "/"} break
	set Var($ElementCount) $element
	if {[lsearch -exact [pamf info items] $element] == -1} {
	    puts "$element not found in PAMF database"
	    close $in
	    return
	}
	if {[lsearch -exact [pamf info item $element] NUMERIC] == -1} {
	    set Type($ElementCount) CODE
	} else {
	    set Type($ElementCount) NUMBER
	}
	set CurrentElement($ElementCount) "zzzzzzz"
	incr ElementCount
    }

    #
    # Now loop through the rest of the rule file and get the
    # data records
    #
    while {[gets $in record] != -1} {
	regsub -all {[ ]+} $record " " newrecord
	set Terminal [lrange $record [expr $ElementCount + 1] end]
	parse [lrange $newrecord 0 [expr $ElementCount + 1]] 0
    }
    close $in
    for {set i 0} {$i < $ElementCount} {incr i} {
	append Rule "):0"
    }
    set Rule
}


#
# The previous proc defintions are the workhorse.
# Now, call these routines and write the output into
# a file that can be sourced by TCL-SDB
#

set out [open "succession.tcl" "w"]
puts $out "#
# PAMF Succession rules - SDB format
#
# Automatically converted from HSG format by mk_rule.tcl
#
# Tom Moore, October 26, 1995
#"
puts $out "set NaturalSuccession \\
    [list [extractRule "nat_succ.dat"]]"
puts $out "set RegenResponse \\
    [list [extractRule "regnsucc.dat"]]"
puts $out "list NaturalSuccession RegenResponse"
close $out
